@@ -27,6 +27,7 @@ By design, hotdoc only supports html output.
 import os
 import html
 import re
+import json
 import urllib.parse
 from urllib.request import urlretrieve
 import shutil
@@ -139,11 +140,16 @@ class Formatter(Configurable):
     pages.
     """
     theme_path = None
+    theme_meta = {}
     extra_theme_path = None
     add_anchors = False
     number_headings = False
     engine = None
     ext_engine = None
+    all_scripts = set()
+    all_stylesheets = set()
+    get_extra_files_signal = Signal()
+    initialized = False
 
     def __init__(self, extension):
         """
@@ -185,8 +191,6 @@ class Formatter(Configurable):
                           ConstantSymbol, ExportedVariableSymbol, AliasSymbol,
                           CallbackSymbol]
 
-        self.all_scripts = set()
-        self.all_stylesheets = set()
         self._docstring_formatter = self._make_docstring_formatter()
         self._current_page = None
         self.extra_assets = None
@@ -194,7 +198,6 @@ class Formatter(Configurable):
         self.number_headings = False
         self.writing_page_signal = Signal()
         self.formatting_page_signal = Signal()
-        self.get_extra_files_signal = Signal()
         self.formatting_symbol_signal = Signal()
         self._order_by_parent = False
 
@@ -271,7 +274,7 @@ class Formatter(Configurable):
 
         extra_files = self._get_extra_files()
 
-        for ex_files in self.get_extra_files_signal(self):
+        for ex_files in Formatter.get_extra_files_signal(self):
             extra_files.extend(ex_files)
 
         for src, dest in extra_files:
@@ -285,13 +288,6 @@ class Formatter(Configurable):
             elif os.path.isdir(src):
                 recursive_overwrite(src, dest)
 
-    def patch_page(self, page, symbol):
-        """
-        Subclasses should implement this in order to allow
-        live patching of the documentation.
-        """
-        raise NotImplementedError
-
     def format_page(self, page):
         """
         Banana banana
@@ -407,7 +403,7 @@ class Formatter(Configurable):
                     else:
                         warn('bad-local-link',
                              "Empty anchor link to %s in %s points nowhere" %
-                             (href, page.source_file))
+                             (href, page.name))
                         link.text = "FIXME broken link to %s" % href
                 link.attrib["href"] = rel_path + href
 
@@ -429,7 +425,7 @@ class Formatter(Configurable):
         src = asset.attrib.get('src')
         if not src:
             warn('no-image-src',
-                 'Empty image source in %s' % page.source_file)
+                 'Empty image source in %s' % page.name)
             return
 
         comps = urllib.parse.urlparse(src)
@@ -454,7 +450,7 @@ class Formatter(Configurable):
         warn('bad-image-src',
              ('In %s, a local assets refers to an unknown source (%s). '
               'It should be available in one of these locations: %s') %
-             (page.source_file, src, str(folders)))
+             (page.name, src, str(folders)))
 
     def write_out(self, page, xml_subpages, output):
         """Banana banana
@@ -490,9 +486,7 @@ class Formatter(Configurable):
         full_path = os.path.join(self.__cache_dir,
                                  self.get_output_folder(page),
                                  page.link.ref)
-        if not os.path.exists(os.path.dirname(full_path)):
-            os.makedirs(os.path.dirname(full_path))
-
+        os.makedirs(os.path.dirname(full_path), exist_ok=True)
         with open(full_path, 'w', encoding='utf-8') as _:
             _.write(page.detailed_description)
 
@@ -513,10 +507,7 @@ class Formatter(Configurable):
 
     def _format_link(self, link, attrs, title):
         out = ''
-        if not link:
-            assert link
-            print("Issue here plz check", title)
-            return title
+        assert link
 
         template = self.get_template('link.html')
         out += '%s' % template.render({'link': link,
@@ -682,6 +673,10 @@ class Formatter(Configurable):
 
     # pylint: disable=too-many-locals
     def _format_page(self, page):
+        redirect = page.meta.get("redirect")
+        if redirect:
+            return ('<meta http-equiv="refresh" content="0; url=%s"/>'  % redirect, True)
+
         symbols_details = []
         by_sections = []
         by_section_symbols = namedtuple('BySectionSymbols',
@@ -728,12 +723,11 @@ class Formatter(Configurable):
         stylesheets_basenames = [os.path.basename(stylesheet)
                                  for stylesheet in stylesheets]
 
-        self.all_stylesheets.update(stylesheets)
-        self.all_scripts.update(scripts)
+        Formatter.all_stylesheets.update(stylesheets)
+        Formatter.all_scripts.update(scripts)
 
         out = template.render(
             {'page': page,
-             'source_file': os.path.basename(page.source_file),
              'scripts': scripts_basenames,
              'stylesheets': stylesheets_basenames,
              'rel_path': rel_path,
@@ -875,6 +869,7 @@ class Formatter(Configurable):
     def _format_interface_symbol(self, interface):
         hierarchy = self._format_hierarchy(interface)
         template = self.get_template('interface.html')
+        interface.extension_attributes['is_section_head'] = self._order_by_parent
         return template.render({'symbol': interface,
                                 'hierarchy': hierarchy})
 
@@ -953,7 +948,7 @@ class Formatter(Configurable):
 
     def _format_comment(self, comment, link_resolver):
         return self._docstring_formatter.translate_comment(
-            comment, link_resolver)
+            comment, link_resolver, self.extension.project.tree)
 
     def __get_theme_files(self, path):
         res = []
@@ -969,16 +964,16 @@ class Formatter(Configurable):
     def _get_extra_files(self):
         res = []
 
-        if self.theme_path:
-            res.extend(self.__get_theme_files(self.theme_path))
-        if self.extra_theme_path:
-            res.extend(self.__get_theme_files(self.extra_theme_path))
+        if Formatter.theme_path:
+            res.extend(self.__get_theme_files(Formatter.theme_path))
+        if Formatter.extra_theme_path:
+            res.extend(self.__get_theme_files(Formatter.extra_theme_path))
 
-        for script_path in self.all_scripts:
+        for script_path in Formatter.all_scripts:
             dest = os.path.join('js', os.path.basename(script_path))
             res.append((script_path, dest))
 
-        for stylesheet_path in self.all_stylesheets:
+        for stylesheet_path in Formatter.all_stylesheets:
             dest = os.path.join('css', os.path.basename(stylesheet_path))
             res.append((stylesheet_path, dest))
 
@@ -1050,24 +1045,30 @@ class Formatter(Configurable):
 
     def parse_toplevel_config(self, config):
         """Parse @config to setup @self state."""
-        html_theme = config.get('html_theme', 'default')
+        if not Formatter.initialized:
+            html_theme = config.get('html_theme', 'default')
 
-        if html_theme != 'default':
-            uri = urllib.parse.urlparse(html_theme)
-            if not uri.scheme:
-                html_theme = config.get_path('html_theme')
-                debug("Using theme located at %s" % html_theme)
-            elif uri.scheme.startswith('http'):
-                html_theme = self.__download_theme(uri)
+            if html_theme != 'default':
+                uri = urllib.parse.urlparse(html_theme)
+                if not uri.scheme:
+                    html_theme = config.get_path('html_theme')
+                    debug("Using theme located at %s" % html_theme)
+                elif uri.scheme.startswith('http'):
+                    html_theme = self.__download_theme(uri)
 
-        if html_theme == 'default':
-            default_theme = os.path.join(HERE, os.pardir,
-                                         'hotdoc_bootstrap_theme', 'dist')
+            if html_theme == 'default':
+                default_theme = os.path.join(HERE, os.pardir,
+                                             'hotdoc_bootstrap_theme', 'dist')
 
-            html_theme = os.path.abspath(default_theme)
-            debug("Using default theme")
+                html_theme = os.path.abspath(default_theme)
+                debug("Using default theme")
+
+            theme_meta_path = os.path.join(html_theme, 'theme.json')
+
+            if os.path.exists(theme_meta_path):
+                with open(theme_meta_path, 'r') as _:
+                    Formatter.theme_meta = json.loads(_.read())
 
-        if Formatter.engine is None:
             searchpath = []
             self.__load_theme_templates(searchpath, HERE)
 
@@ -1085,6 +1086,8 @@ class Formatter(Configurable):
                 extensions=[CoreExtension(), CodeExtension()])
             Formatter.engine.global_vars.update({'e': html.escape})
 
+            Formatter.initialized = True
+
     def get_template(self, name):
         """
         Banana banana
@@ -1119,5 +1122,10 @@ class Formatter(Configurable):
         except IOError:
             return None
 
-        return etree.XML(template.render({'page': page,
-                                          'subpages': subpages}))
+        ret = etree.XML(template.render({'page': page,
+                                         'subpages': subpages}))
+        assets = ret.xpath('.//*[@src]')
+        for asset in assets:
+            self.__lookup_asset(asset, self.extension.project, page)
+
+        return ret
